--[[
Bison - a framework for free positioning of buffs.

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to:
	
Free Software Foundation, I.,
51 Franklin Street, Fifth Floor,
Boston, MA  02110-1301, USA.
--]]

local Type, Version = 'BisonBuff', 2
local AceGUI = LibStub( 'AceGUI-3.0')

UIFrameFlashStop = UIFrameFlashRemoveFrame or UIFrameFlashStop

--[[-----------------------------------------------------------------------------
Scripts
-------------------------------------------------------------------------------]]
local function Control_OnMouseDown( frame)
	frame.obj:Fire( 'OnMouseDown')
end

local function Control_OnMouseUp( frame)
	frame.obj:Fire( 'OnMouseUp')
end

local function Control_OnEnter( frame)
	frame.obj:Fire( 'OnEnter')
end

local function Control_OnLeave( frame)
	frame.obj:Fire( 'OnLeave')
end

--[[-----------------------------------------------------------------------------
Methods
-------------------------------------------------------------------------------]]
local methods = {
	['OnAcquire'] = function( self)
	end,

	['OnRelease'] = function( self)
		UIFrameFlashStop( self.frame)
	end,

	['SetLock'] = function( self, locked)
		self.userdata.locked = locked
		if locked or self.id == 0 then
			self.frame:SetScript( 'OnMouseDown', nil)
			self.frame:SetScript( 'OnMouseUp',   nil)
			if not self.userdata.name then
				self.frame:Hide()
			end
			self.ghostlabel:Hide()
			self.ghosticon:Hide()
		else
			self.frame:SetScript( 'OnMouseDown', Control_OnMouseDown)
			self.frame:SetScript( 'OnMouseUp',   Control_OnMouseUp)
			self.frame:Show()
			self.ghostlabel:Show()
			self.ghosticon:Show()
		end
	end,

	['UpdateLeft'] = function( self, flashing)
		local user = self.userdata
		if user.text then
			self.left:SetText( user.text)
			if user.warn then
				self.left:SetVertexColor( HIGHLIGHT_FONT_COLOR.r, HIGHLIGHT_FONT_COLOR.g, HIGHLIGHT_FONT_COLOR.b);
			else
				self.left:SetVertexColor( NORMAL_FONT_COLOR.r, NORMAL_FONT_COLOR.g, NORMAL_FONT_COLOR.b);
			end
			self.left:Show()
			self:Fire( 'OnTooltip')
		else
			self.left:Hide()
		end
		local shouldFlash = flashing and user.timeLeft
		if shouldFlash then
			if user.duration and user.duration < BUFF_WARNING_TIME then
				shouldFlash = user.timeLeft < user.duration/2
			else
				shouldFlash = user.timeLeft < BUFF_WARNING_TIME
			end
		end
		if shouldFlash then
			if not UIFrameIsFading( self.frame) then
				UIFrameFlash( self.frame, 0.2, 0.2, user.timeLeft, true, 0, 0.7)
			end
		else
			if UIFrameIsFading( self.frame) then
				UIFrameFlashStop( self.frame)
			end
			self.frame:SetAlpha( 1)
		end
	end,

	['UpdateTexture'] = function( self)
		local user = self.userdata
		if user.texture then
			self.icon:SetTexture( user.texture)
			if user.textcoord then
				self.icon:SetTexCoord( unpack( user.textcoord))
			end
			self.icon:Show()
		else
			self.icon:Hide()
		end
	end,

	['UpdateCount'] = function( self)
		local user = self.userdata
		if user.count and user.count > 1 then
			self.count:SetText( user.count)
			self.count:Show()
		else
			self.count:Hide()
		end
	end,

	['UpdateSymbol'] = function( self)
		if ENABLE_COLORBLIND_MODE == '1' then
			local user = self.userdata
			local text = user.type and DebuffTypeSymbol[user.type]
			if text then
				self.symbol:SetText( text)
				self.symbol:Show()
			else
				self.symbol:Hide()
			end
		else
			self.symbol:Hide()
		end
	end,

	['UpdateBorder'] = function( self)
		local user = self.userdata
		local color = user.type and DebuffTypeColor[user.type]
		if color then
			self.border:SetVertexColor( color.r, color.g, color.b, 1)
		else
			self.border:SetVertexColor( 0, 0, 0, 0)
		end
	end,

	['Initialize'] = function( self, id, color, filter)
		self.id     = id
		local frame = self.frame
		frame:SetAlpha( 1)
		if filter == 'HELPFUL' then
			self.border:Hide()
			self.ghostlabel:SetText( tostring( id))
		elseif filter == 'HARMFUL' then
			frame:SetScript( 'OnClick', nil)
			self.border:SetTexture( 'Interface\\Buttons\\UI-Debuff-Overlays')
			self.border:SetTexCoord( 0.296875, 0.5703125, 0, 0.515625)
			self.border:Show()
			self.ghostlabel:SetText( tostring( id))
		else
			self.border:SetTexture( 'Interface\\Buttons\\UI-TempEnchant-Border')
			self.border:SetTexCoord( 0, 1, 0, 1)
			self.border:Show()
			self.ghostlabel:SetText( (id == 1) and 'MH' or 'OH')
		end
		self.ghosticon:SetTexture( 0, 0, 1, 1)
	end,

	['UpdateAll'] = function( self, flashing)
		local user = self.userdata
		if user.name or not user.locked then
			self:UpdateTexture()
			self:UpdateCount()
			self:UpdateSymbol()
			self:UpdateBorder()
			self:UpdateLeft( flashing)
			self.frame:Show()
		else
			self.frame:Hide()
			if GameTooltip:IsOwned( self.frame) then
				GameTooltip:Hide()
			end
		end
	end,
}

--[[-----------------------------------------------------------------------------
Constructor
-------------------------------------------------------------------------------]]
local function Constructor()
	local num  = AceGUI:GetNextWidgetNum( Type)
	local name = Type .. num
	
	local frame = CreateFrame( 'Button', name, UIParent)
	frame:SetWidth( 32)
	frame:SetHeight( 32)
	frame:EnableMouse( true)
	frame:SetScript( 'OnEnter', Control_OnEnter)
	frame:SetScript( 'OnLeave', Control_OnLeave)
	frame:Hide()
	
	local icon = frame:CreateTexture( name..'Icon', 'BACKGROUND')
	icon:SetPoint( 'TOPLEFT', frame, 'TOPLEFT', 0, 0)
	icon:SetPoint( 'BOTTOMRIGHT', frame, 'BOTTOMRIGHT', 0, 0)
	
	local count = frame:CreateFontString( name..'Count', 'BACKGROUND', 'NumberFontNormal')
	count:SetPoint( 'BOTTOMRIGHT', frame, 'BOTTOMRIGHT', -2, 2)

	local left = frame:CreateFontString( name..'Name', 'ARTWORK', 'GameFontNormalSmall')
	left:SetPoint( 'TOP', frame, 'BOTTOM')

	local symbol = frame:CreateFontString( name..'Symbol', 'BACKGROUND', 'NumberFontNormal')
	symbol:SetPoint( 'TOPLEFT', frame, 'TOPLEFT', 2, -2)
	symbol:Hide()
	
	local border = frame:CreateTexture( name..'Border', 'OVERLAY')
	border:SetPoint( 'TOPLEFT', frame, 'TOPLEFT', 0, 0)
	border:SetPoint( 'BOTTOMRIGHT', frame, 'BOTTOMRIGHT', 0, 0)
	border:Hide()

	local ghostlabel = frame:CreateFontString( nil, 'OVERLAY', 'GameFontHighlight')
	ghostlabel:SetHeight( 10)
	ghostlabel:SetPoint( 'TOPLEFT', frame, 'TOPLEFT', 0, 0)
	ghostlabel:SetPoint( 'BOTTOMRIGHT', frame, 'BOTTOMRIGHT', 0, 0)
	ghostlabel:Hide()

	local ghosticon = frame:CreateTexture( nil, 'OVERLAY')
	ghosticon:SetBlendMode( 'ADD')
	ghosticon:SetPoint( 'TOPLEFT', frame, 'TOPLEFT', 0, 0)
	ghosticon:SetPoint( 'BOTTOMRIGHT', frame, 'BOTTOMRIGHT', 0, 0)
	ghosticon:Hide()
	
	local widget = {
		num    = num,
		frame  = frame,
		icon   = icon,
		count  = count,
		left   = left,
		symbol = symbol,
		border = border,
		ghostlabel = ghostlabel,
		ghosticon  = ghosticon,
		type  = Type
	}
	for method, func in pairs(methods) do
		widget[method] = func
	end
	frame.obj = widget
	
	return AceGUI:RegisterAsWidget( widget)
end

AceGUI:RegisterWidgetType( Type, Constructor, Version)
