--[[
Bison - a framework for free positioning of buffs.

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to:
	
Free Software Foundation, I.,
51 Franklin Street, Fifth Floor,
Boston, MA  02110-1301, USA.
--]]

local Bison = LibStub( 'AceAddon-3.0'):GetAddon( 'Bison')
local L = LibStub( 'AceLocale-3.0'):GetLocale( 'Bison')

local Module = Bison:NewModule( 'Options', 'AceEvent-3.0', 'AceConsole-3.0', 'LibDebugLog-1.0')

--[[---------------------------------------------------------------------------------
Local
------------------------------------------------------------------------------------]]
local function OpenToCategory( appName, ...)
	local key = appName
	for n = 1,select( '#', ...) do
		key = key..'\001'..select( n, ...)
	end
	local grp = LibStub( 'AceConfigDialog-3.0').BlizOptions[appName]
	if grp and grp[key] then
		InterfaceOptionsFrame_OpenToCategory( grp[key].frame)
	end
end

local function OnCommand( input)
	if input and input:trim() ~= '' then
		LibStub( 'AceConfigCmd-3.0'):HandleCommand( 'bison', 'bison', input)
	else
		OpenToCategory( 'bison', 'profile')
		OpenToCategory( 'bison', 'main')
	end
end

local function IsEnabled()
	return Bison:IsEnabled()
end

local function SetEnable( info, value) 
	if value ~= Bison:IsEnabled() then
		if value then
			Bison:Enable()
		else
			Bison:Disable()
		end
	end
	Module:SendMessage( 'BISON_UPDATE')
end

local function IsDebug()
	return Bison:IsDebugLogEnabled()
end

local function SetDebug( info, value) 
	Bison:SetDebug( value)
end

function PrintVersion() 
	Bison:Print( Bison.version) 
end

function PrintHelp()
	local function PrintCmd( cmd, desc) Bison:Print( format( SUB_HELP_MSG, cmd, desc)) end
	Bison:Print( 'Commands (/bi, /bison)')
	PrintCmd( 'enable', L.EnabledDesc)
	PrintCmd( 'lock', L.LockDesc)
	PrintCmd( 'debug', L.DebugDesc)
	PrintCmd( '<bar> show', L.ShowDesc)
	PrintCmd( '<bar> flashing', L.FlashingDesc)
	PrintCmd( '<bar> timer', L.TimerDesc)
	PrintCmd( '<bar> scale <value>', L.ScaleDesc)
	PrintCmd( '<bar> rows <value>', L.RowsDesc)
	PrintCmd( '<bar> cols <value>', L.ColsDesc)
	PrintCmd( '<bar> horizontal', L.HorizontalDesc)
	PrintCmd( '<bar> xPadding <value>', L.XPaddingDesc)
	PrintCmd( '<bar> yPadding <value>', L.YPaddingDesc)
	PrintCmd( '<bar> sort <value>', L.SortDesc)
	PrintCmd( 'version', L.VersionDesc)
end

--[[---------------------------------------------------------------------------------
Main
------------------------------------------------------------------------------------]]
local function GetProperty( info)
	local key = info[#info]
	return Bison.db.profile[key]
end

local function SetProperty( info, value)
	local key = info[#info]
	Bison.db.profile[key] = value
	Module:SendMessage( 'BISON_UPDATE')
end

local main = {
	type = 'group', order = 10, name = 'Main', get = GetProperty, set = SetProperty, 
	args = {
		release     = { type = 'description', order = 20,  name = Bison.version, cmdHidden = true, fontSize = 'large' },
		description = { type = 'description', order = 30,  name = L.Description, cmdHidden = true, fontSize = 'large' },
		space1      = { type = 'description', order = 40,  name = ' ', cmdHidden = true },
		enabled     = { type = 'toggle',      order = 140, name = L.EnabledName, desc = L.EnabledDesc,   get = IsEnabled, set = SetEnable, width = 'full' },
		debug       = { type = 'toggle',      order = 150, name = L.DebugName,   desc = L.DebugDesc,     get = IsDebug,   set = SetDebug,  width = 'full' },
		locked      = { type = 'toggle',      order = 160, name = L.LockName,    desc = L.LockDesc,      width = 'full' },
	}
}
local plugins = {}
local options = {
	type = 'group', name = Bison.title, childGroups = 'tab',
	args = {
		version  = { type = 'execute', order = 83, name = 'version', dialogHidden = true, disabled = true, func = PrintVersion },
		help     = { type = 'execute', order = 84, name = 'help',    dialogHidden = true, disabled = true, func = PrintHelp },
	},
	plugins = plugins,
}

--[[---------------------------------------------------------------------------------
Class
------------------------------------------------------------------------------------]]
function Module:OnInitialize()
	local profile = LibStub( 'AceDBOptions-3.0'):GetOptionsTable( Bison.db)
	profile.order = 9999
	profile.args.desc.dialogHidden = true
	options.args.profile = profile
	self:AddPlugin( 'pluginmain', { main = main })
	LibStub( 'AceConfig-3.0'):RegisterOptionsTable( 'bison', options)
	local dialog = LibStub( 'AceConfigDialog-3.0')
	dialog:AddToBlizOptions( 'bison', 'Bison', nil, 'main')
	dialog:AddToBlizOptions( 'bison', L.Profile,   'Bison', 'profile')
	Bison:RegisterChatCommand( 'bison', OnCommand)
	Bison:RegisterChatCommand( 'bi', OnCommand)
end

function Module:OnEnable()
end

function Module:OnDisable()
end

function Module:AddPlugin( name, arr)
	if type( arr) == 'table' then
		plugins[name] = arr
	end
end
